#ifndef __ema_tutorial_niprovider_h_
#define __ema_tutorial_niprovider_h_

#include <string>
#include "Ema.h"
#include "ConnectionStateListener.h"

namespace thomsonreuters
{
namespace tutorials
{
namespace ema
{
namespace nip
{
	//--------------------------------------------------------------------------------
	// This class is the main interface between the Ni Provider application and EMA.
	// It defines the high level methods used by the main workflow.You can use its 
	// methods has a reference to better understand how to use EMA for performing 
	// specific actions like listening to connection messages or updating an item.
	// This class will be completed by more methods as we progress in the next 
	// tutorials.
	class NiProvider {

	public:

		NiProvider();
		virtual ~NiProvider();

		//--------------------------------------------------------------------------------
		// This method connects the Non Interactive Provider to the ADH TREP component.
		// To this aim, is uses the <host> and <userName> harcoded variables that you 
		// must change to match your actual TREP infrastructure before you build and run
		// this application
		void connect();

		//--------------------------------------------------------------------------------
		// This method disconnects from the ADH TREP component.
		void disconnect();

		//--------------------------------------------------------------------------------
		// This method allows to know if the NiProvider is connected or not to the ADH.
		// It returns true true if it's connected, false otherwise
		bool isConnected() const;

	private:

		thomsonreuters::ema::access::OmmProvider * _provider = 0;
		ConnectionStateListener _connectionStateListener;
	};

}
}
}
}

#endif // __ema_tutorial_niprovider_h_
