#include "NiProvider.h"
#include <iostream>

using namespace std;
using namespace thomsonreuters::ema::access;
using namespace thomsonreuters::ema::rdm;
using namespace thomsonreuters::tutorials::ema::nip;

//--------------------------------------------------------------------------------

NiProvider::NiProvider()
{
	cout << "  Provider created" << endl;
}

//--------------------------------------------------------------------------------

NiProvider::~NiProvider()
{
	disconnect();
}

//--------------------------------------------------------------------------------

void NiProvider::connectAs(const std::string & userName)
{

	// Exit the method if already connected
	if (_provider != 0)
		return;

	cout << "  Connecting Provider to the ADH as " << userName << endl;

	// Create a Non Interactive Provider using harcoded parameters
	_provider = new OmmProvider(
						OmmNiProviderConfig()
						.username(userName.c_str()));

	// Register the _connectionStateListener to receive login messages from EMA
	_provider->registerClient(
						ReqMsg()
						.domainType(MMT_LOGIN)
						.name(userName.c_str()),
						_connectionStateListener,
						(void*)0);
}

//--------------------------------------------------------------------------------

void NiProvider::disconnect()
{
	// Exit the method if already disconnected
	if (_provider == 0)
		return;

	cout << "  Disconnecting..." << endl;

	// Disconnect from the infrastructure and automatically unregister the 
	// _connectionStateListener we registered in the connect() method to 
	// receive login messages from EMA
	delete _provider;
	_provider = 0;
}

//--------------------------------------------------------------------------------

bool NiProvider::isConnected() const 
{
	return _connectionStateListener.isConnected();
}

//--------------------------------------------------------------------------------

void NiProvider::refresh(const std::string & serviceName, const std::string & itemName)
{
	if (!isConnected())
	{
		cout << "  ERROR: Can't refresh " << itemName << ". The provider is not connected." << endl;
		return;
	}

	UInt64 itemHandle = 1;

	cout << "  Refreshing " << itemName << endl;

	_provider->submit(
					RefreshMsg()
					.serviceName(serviceName.c_str())
					.name(itemName.c_str())
					.domainType(MMT_MARKET_PRICE)
					.state(
						OmmState::OpenEnum,
						OmmState::OkEnum,
						OmmState::NoneEnum,
						"UnSolicited Refresh Completed")
					.payload(FieldList()
						.addAscii(3, "My Company Share")				// DSPLY_NAME
						.addReal(22, 14400, OmmReal::ExponentNeg2Enum)	// BID
						.addReal(25, 14700, OmmReal::ExponentNeg2Enum)	// ASK
						.addReal(19, 14100, OmmReal::ExponentNeg2Enum)	// OPEN_PRC
						.addReal(21, 14900, OmmReal::ExponentNeg2Enum)	// HST_CLOSE
						.complete())
					.complete(),
					itemHandle);
}

//--------------------------------------------------------------------------------

