// *|-----------------------------------------------------------------------------
// *|            This source code is provided under the Apache 2.0 license      --
// *|  and is provided AS IS with no warranty or guarantee of fit for purpose.  --
// *|                See the project's LICENSE.md for details.                  --
// *|           Copyright Thomson Reuters 2016. All rights reserved.            --
// *|-----------------------------------------------------------------------------

// *|-----------------------------------------------------------------------------
// *|                   Non Interactive Provider EMA Tutorial                   --
// *|                                                                           --
// *|                 Tutorial 6 - Publishing a Source Directory                --
// *|                                                                           --
// *| This tutorial is the 6th part of a series of tutorials with the goal      --
// *| of creating a Non Iteractive Provider using the C++ edition of EMA, the   --
// *| Elektron Message API. See the README.md for more details.                 --
// *|-----------------------------------------------------------------------------

#include <iostream>

#ifdef WIN32
#include <windows.h>
#else
#include <sys/time.h>
#endif

#include "NiProvider.h"

//--------------------------------------------------------------------------------

using namespace std;
using namespace thomsonreuters::tutorials::ema::nip;

//--------------------------------------------------------------------------------
// This function blocks the current thread by sleeping the number of seconds
// indicated by the "seconds" argument. It exits when the time is elapsed.
void waitFor(int millisecs);

//--------------------------------------------------------------------------------
// This function implements the main workflow of the application. Actual Elektron
// Message API calls are delegated to the NiProvider class.
int main(int argc, char* argv[])
{
	try
	{
		cout << "-------------------------------------------------------------------------------" << endl;
		cout << "|                    Non Interactive Provider EMA Tutorial                    |" << endl;
		cout << "|                                                                             |" << endl;
		cout << "|                  Tutorial 6 - Publishing a Source Directory                 |" << endl;
		cout << "-------------------------------------------------------------------------------" << endl;
		
		NiProvider provider;

		provider.connectAs("YOUR_PROVIDER_USER_NAME");

		waitFor(5);

		provider.refresh("TEST_NI_PUB", "SHARE-0");

		waitFor(60);

		provider.disconnect();
	}
	catch (const thomsonreuters::ema::access::OmmException& excp)
	{
		cout << excp << endl;
	}

	cout << "  Exiting the application" << endl;
	return 0;
}

//--------------------------------------------------------------------------------

void waitFor(int seconds)
{
	cout << "  Waiting for " << seconds << " seconds..." << endl;

#if defined WIN32
	::Sleep((DWORD)(seconds * 1000));
#else
	struct timespec sleeptime;
	sleeptime.tv_sec = seconds;
	sleeptime.tv_nsec = 0;
	nanosleep(&sleeptime, 0);
#endif
}

//--------------------------------------------------------------------------------
