#ifndef __ema_tutorial_niprovider_h_
#define __ema_tutorial_niprovider_h_

#include <string>
#include "Ema.h"
#include "ConnectionStateListener.h"
#include "Item.h"

namespace thomsonreuters
{
namespace tutorials
{
namespace ema
{
namespace nip
{
	//--------------------------------------------------------------------------------
	// This class is the main interface between the Ni Provider application and EMA.
	// It defines the high level methods used by the main workflow.You can use its 
	// methods has a reference to better understand how to use EMA for performing 
	// specific actions like listening to connection messages or updating an item.
	// This class will be completed by more methods as we progress in the next 
	// tutorials.
	class NiProvider {

	public:

		NiProvider();
		virtual ~NiProvider();

		//--------------------------------------------------------------------------------
		// Sets the service name used by this provider to publish messages
		void setServiceName(const std::string & serviceName);

		//--------------------------------------------------------------------------------
		// Returns the service name used by this provider to publish messages
		const std::string & getServiceName() const;

		//--------------------------------------------------------------------------------
		// This method connects the Non Interactive Provider to the ADH TREP component
		// and logs in using the <userName> parameter. 
		// The IP address and port number of the ADH are set in EmaConfig.xml files that 
		// must be changed to match your TREP infrastructure.
		void connectAs(const std::string & userName);

		//--------------------------------------------------------------------------------
		// This method disconnects from the ADH TREP component.
		void disconnect();

		//--------------------------------------------------------------------------------
		// This method allows to know if the NiProvider is connected or not to the ADH.
		// It returns true true if it's connected, false otherwise
		bool isConnected() const;

		//--------------------------------------------------------------------------------
		// This method sends a MarketPrice refresh message for the data item identified by
		// the <itemName> parameters.
		void refresh(const std::string & itemName);

		//--------------------------------------------------------------------------------
		// This method sends a MarketPrice update message for the data item identified by
		// the <itemName> parameters.
		void update(const std::string & itemName);

	private:

		// Field IDs and Acronyms used by this class
		enum Fields 
		{
			DSPLY_NAME = 3,
			BID = 22,
			ASK = 25,
			OPEN_PRC = 19,
			HST_CLOSE = 21
		};

		thomsonreuters::ema::access::OmmProvider * _provider = 0;
		ConnectionStateListener _connectionStateListener;
		std::string _serviceName = "NI_PUB";
		Item _theOnlyOneItem;
	};

}
}
}
}

#endif // __ema_tutorial_niprovider_h_
