#include "Item.h"

#include <iostream>

using namespace std;
using namespace thomsonreuters::tutorials::ema::nip;

//--------------------------------------------------------------------------------

int Item::_nextHandle = 1;

//--------------------------------------------------------------------------------

Item::Item()
	:_name("Unknown")
	, _displayName(_name + " Corp International")
	, _basePrice(10)
	, _priceDistribution(0, 1)
{
}

//--------------------------------------------------------------------------------

Item::Item(const std::string & name, int basePrice)
	:_name(name)
	,_displayName(_name + " Corp International")
	, _basePrice(basePrice)
	,_priceDistribution(0, 1)
{
	_handle = _nextHandle++;
	initRandomizer();
	generateOpenAndClosePrices();
	generateNextTick();
}

//--------------------------------------------------------------------------------

Item::Item(const Item & toCopy)
{
	copy(toCopy);
}

//--------------------------------------------------------------------------------

Item::~Item()
{
}


//--------------------------------------------------------------------------------

Item & Item::operator=(const Item & toCopy)
{
	if (&toCopy != this)
	{
		copy(toCopy);
	}

	return *this;
}

//--------------------------------------------------------------------------------

void Item::copy(const Item & toCopy)
{
	_handle = toCopy._handle;
	_name = toCopy._name;
	_displayName = toCopy._displayName;
	_basePrice = toCopy._basePrice;
	_openPrice = toCopy._openPrice;
	_closePrice = toCopy._closePrice;
	_bidPrice = toCopy._bidPrice;
	_askPrice = toCopy._askPrice;
	_priceDistribution = toCopy._priceDistribution;
}

//--------------------------------------------------------------------------------

const std::string & Item::getName() const
{
	return _name;
}

//--------------------------------------------------------------------------------

int Item::getHandle() const
{
	return _handle;
}

//--------------------------------------------------------------------------------

const std::string & Item::getDisplayName() const
{
	return _displayName;
}

//--------------------------------------------------------------------------------

int Item::getOpenPrice() const
{
	return _openPrice;
	
}

//--------------------------------------------------------------------------------

int Item::getClosePrice() const
{
	return _closePrice;
}

//--------------------------------------------------------------------------------

int Item::getBidPrice() const
{
	return _bidPrice;
}

//--------------------------------------------------------------------------------

int Item::getAskPrice() const
{
	return _askPrice;
}

//--------------------------------------------------------------------------------

void Item::generateNextTick()
{
	_bidPrice = _priceDistribution(_randomDevice);
	_askPrice = _priceDistribution(_randomDevice);

	// Swapp prices if needed (ASK must be greater than BID) 
	if (_bidPrice > _askPrice)
	{
		int newBidPrice = _askPrice;
		_askPrice = _bidPrice;
		_bidPrice = newBidPrice;
	}
}

//--------------------------------------------------------------------------------

void Item::generateOpenAndClosePrices() 
{
	_openPrice = _priceDistribution(_randomDevice);
	_closePrice = _priceDistribution(_randomDevice);
}

//--------------------------------------------------------------------------------

void Item::initRandomizer()
{

	// Generates a random spread between 5% and 20% of the base price
	uniform_int_distribution<int> spreadPercentageDistribution(5, 20);
	int spreadPercentage = spreadPercentageDistribution(_randomDevice);
	int spread = _basePrice * spreadPercentage / 100;

	// Calculate the min and max values
	int minPrice = _basePrice - spread / 2;
	int maxPrice = _basePrice + spread / 2;

	_priceDistribution = uniform_int_distribution<int>(minPrice, maxPrice);
}

