#ifndef __ema_tutorial_niprovider_item_h_
#define __ema_tutorial_niprovider_item_h_

#include <string>
#include <random>
#include <functional>
#include "Ema.h"

namespace thomsonreuters 
{
namespace tutorials 
{
namespace ema 
{
namespace nip 
{
	//--------------------------------------------------------------------------------
	// This class is the source of data for the provider it simulates a MarketPrice
	// live data item made of 5 fields.
	class Item
	{
	public:

		//--------------------------------------------------------------------------------
		// Builds a MarketPrice data item. Default constructor.
		Item();

		//--------------------------------------------------------------------------------
		// Builds a MarketPrice data item. <name> is the name (aka. RIC) of the item. 
		// <basePrice> is a base price used to randomly generate the initial fields values
		// as well as the ticks for the item updates.
		Item(const std::string & name, int basePrice);

		//--------------------------------------------------------------------------------
		// Builds a MarketPrice data item. Copy constructor.
		Item(const Item & toCopy);

		//--------------------------------------------------------------------------------
		// Cleans-up the item.
		virtual ~Item();

		//--------------------------------------------------------------------------------
		// Copies a MarketPrice data item. Assignment operator.
		Item & operator=(const Item & toCopy);

		//--------------------------------------------------------------------------------
		// Returns the item name (aka. RIC).
		const std::string & getName() const;

		//--------------------------------------------------------------------------------
		// Returns the unique item handle.
		int getHandle() const;

		//--------------------------------------------------------------------------------
		// Returns the Display Name field's value.
		const std::string & getDisplayName() const;

		//--------------------------------------------------------------------------------
		// Returns the Open Price field's value.
		int getOpenPrice() const;

		//--------------------------------------------------------------------------------
		// Returns the Close Price field's value.
		int getClosePrice() const;

		//--------------------------------------------------------------------------------
		// Returns the Bid field's value.
		int getBidPrice() const;

		//--------------------------------------------------------------------------------
		// Returns the Ask field's value.
		int getAskPrice() const;

		//--------------------------------------------------------------------------------
		// Generates the initial fields' values.
		void generateOpenAndClosePrices();

		//--------------------------------------------------------------------------------
		// Generates new Bid & Ask fields' values to simulate a tick.
		void generateNextTick();

	private:

		void initRandomizer();

		static int _nextHandle;

		int _handle;
		std::string _name;
		std::string _displayName;
		int _basePrice;
		int _openPrice;
		int _closePrice;
		int _bidPrice;
		int _askPrice;

		std::random_device _randomDevice;
		std::uniform_int_distribution<int> _priceDistribution;

		void copy(const Item & toCopy);
	};

}
}
}
}

#endif // __ema_tutorial_niprovider_h_
